import { addDecimals, toIntegerString } from '@vertex-protocol/utils';
import { subaccountToHex } from '../utils/bytes32';
/**
 * Returns the EIP712 compatible values for signing.
 *
 * Note: We use the string representation of bigint to be compatible with JSON.stringify
 *
 * @param requestType
 * @param params
 */
export function getVertexEIP712Values(requestType, params) {
    // Typescript does not yet support type narrowing + generic lookup types, hence the hacks here
    let values;
    switch (requestType) {
        case 'withdraw_collateral':
            values = getWithdrawCollateralValues(params);
            break;
        case 'mint_lp':
            values = getMintLpValues(params);
            break;
        case 'burn_lp':
            values = getBurnLpValues(params);
            break;
        case 'place_order':
            values = getOrderValues(params);
            break;
        case 'place_isolated_order':
            values = getIsolatedOrderValues(params);
            break;
        case 'list_trigger_orders':
            values = getListTriggerOrdersValues(params);
            break;
        case 'cancel_orders':
            values = getOrderCancellationValues(params);
            break;
        case 'cancel_product_orders':
            values = getProductOrdersCancellationValues(params);
            break;
        case 'liquidate_subaccount':
            values = getLiquidateSubaccountValues(params);
            break;
        case 'link_signer':
            values = getLinkSignerValues(params);
            break;
        case 'transfer_quote':
            values = getTransferQuoteValues(params);
            break;
        case 'leaderboard_authentication':
            values = getLeaderboardAuthenticationValues(params);
            break;
        case 'mint_vlp':
            values = getMintVlpValues(params);
            break;
        case 'burn_vlp':
            values = getBurnVlpValues(params);
            break;
        default:
            throw new Error(`Unsupported request type: ${requestType}`);
    }
    return values;
}
function getWithdrawCollateralValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        productId: params.productId,
        amount: toIntegerString(params.amount),
        nonce: params.nonce,
    };
}
function getMintLpValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        productId: params.productId,
        amountBase: toIntegerString(params.amountBase),
        quoteAmountLow: toIntegerString(params.quoteAmountLow),
        quoteAmountHigh: toIntegerString(params.quoteAmountHigh),
        nonce: params.nonce,
    };
}
function getBurnLpValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        productId: params.productId,
        amount: toIntegerString(params.amount),
        nonce: params.nonce,
    };
}
function getOrderValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        priceX18: toIntegerString(addDecimals(params.price)),
        amount: toIntegerString(params.amount),
        expiration: toIntegerString(params.expiration),
        nonce: params.nonce,
    };
}
function getIsolatedOrderValues(params) {
    return {
        ...getOrderValues(params),
        margin: toIntegerString(params.margin),
    };
}
function getListTriggerOrdersValues(params) {
    return {
        recvTime: toIntegerString(params.recvTime),
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
    };
}
function getOrderCancellationValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        productIds: params.productIds,
        digests: params.digests,
        nonce: params.nonce,
    };
}
function getProductOrdersCancellationValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        productIds: params.productIds,
        nonce: params.nonce,
    };
}
function getLiquidateSubaccountValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        liquidatee: subaccountToHex({
            subaccountOwner: params.liquidateeOwner,
            subaccountName: params.liquidateeName,
        }),
        mode: params.mode,
        healthGroup: toIntegerString(params.healthGroup),
        amount: toIntegerString(params.amount),
        nonce: params.nonce,
    };
}
function getLinkSignerValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        signer: params.signer,
        nonce: params.nonce,
    };
}
function getTransferQuoteValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        recipient: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.recipientSubaccountName,
        }),
        amount: toIntegerString(params.amount),
        nonce: params.nonce,
    };
}
function getLeaderboardAuthenticationValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        expiration: toIntegerString(params.expiration),
    };
}
function getMintVlpValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        quoteAmount: toIntegerString(params.quoteAmount),
        nonce: params.nonce,
    };
}
function getBurnVlpValues(params) {
    return {
        sender: subaccountToHex({
            subaccountOwner: params.subaccountOwner,
            subaccountName: params.subaccountName,
        }),
        vlpAmount: toIntegerString(params.vlpAmount),
        nonce: params.nonce,
    };
}
//# sourceMappingURL=getVertexEIP712Values.js.map