import { toIntegerString } from '@vertex-protocol/utils';
import { getDefaultRecvTime } from './recvTime';
/**
 * Generates an order nonce based on recvTime in milliseconds, defaulting to Date.now() + 90 seconds
 * @param recvTimeMillis
 * @param randomInt a random integer to avoid hash collisions
 */
export function getOrderNonce(recvTimeMillis = getDefaultRecvTime(), randomInt = Math.floor(Math.random() * 1000)) {
    return toIntegerString(getOrderNonceBigInt(recvTimeMillis, randomInt));
}
/**
 * Generates a trigger order nonce based on recvTime in milliseconds, defaulting to Date.now() + 90 seconds
 * Trigger order nonces require that the first bit is set to 1, this differentiates it from a regular order nonce
 *
 * @param recvTimeMillis
 * @param randomInt
 */
export function getTriggerOrderNonce(recvTimeMillis = getDefaultRecvTime(), randomInt = Math.floor(Math.random() * 1000)) {
    const regularOrderNonce = getOrderNonceBigInt(recvTimeMillis, randomInt);
    const triggerOrderNonce = regularOrderNonce | (1n << 63n);
    return toIntegerString(triggerOrderNonce);
}
/**
 * Determines if a nonce is a trigger order nonce
 */
export function isTriggerOrderNonce(orderNonce) {
    const nonceBigInt = BigInt(orderNonce);
    // Check if the 63rd bit is set to 1
    return Boolean(nonceBigInt & (1n << 63n));
}
/**
 * Gets the recvTime in millis from an order nonce
 *
 * @param orderNonce
 */
export function getRecvTimeFromOrderNonce(orderNonce) {
    // We need to trim the first (63rd) bit, which is set to 1 for trigger orders
    const orderNonceBitMasked = BigInt(orderNonce) & ~(1n << 63n);
    const bigIntRecvTime = orderNonceBitMasked >> 20n;
    return Number(bigIntRecvTime.toString());
}
function getOrderNonceBigInt(recvTimeMillis, randomInt) {
    return (BigInt(recvTimeMillis) << 20n) + BigInt(randomInt);
}
//# sourceMappingURL=orderNonce.js.map